# ../scripts/custom/gg_scoreboard/gg_scoreboard.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# Python Imports
#   OS
from os import name as os_name

# Eventscripts Imports
#   ES
from es import exists
from es import getUseridList
from es import ServerVar
#   Gamethread
from gamethread import delayed

# SPE Imports
from spe import getLocVal
from spe import getPlayer
from spe import setLocVal

# GunGame Imports
#   Addons
from gungame51.core.addons.shortcuts import AddonInfo
#   Players
from gungame51.core.players.shortcuts import Player
#   Weapons
from gungame51.core.weapons.shortcuts import get_level_multikill

# =============================================================================
# >> ADDON REGISTRATION/INFORMATION
# =============================================================================
info = AddonInfo()
info.name = 'gg_scoreboard'
info.title = 'GG Scoreboard'
info.author = 'Satoon101'
info.version = '1.9'

# =============================================================================
# >> PUBLIC VARIABLE
# =============================================================================
ServerVar(info.name + '_version', info.version,
        info.title + ' Version').set(info.version)
ServerVar(info.name + '_version').makepublic()

# =============================================================================
# >> GLOBAL VARIABLES
# =============================================================================
gg_scoreboard_multikill = ServerVar('gg_scoreboard_multikill')

gg_kill_offset = 3336 if os_name == 'nt' else 3356

gg_death_offset = 3340 if os_name == 'nt' else 3360


# =============================================================================
# >> LOAD & UNLOAD
# =============================================================================
def load():
    '''Called when the script is loaded'''

    # Set all player scores in case loaded during game
    set_player_scores()


# =============================================================================
# >> GAME EVENTS
# =============================================================================
def gg_levelup(event_var):
    '''Called when a player levels up'''

    # Set leveler's score
    set_score(event_var['userid'])


def gg_leveldown(event_var):
    '''Called when a player levels down'''

    # Set leveler's score
    set_score(event_var['userid'])


def player_spawn(event_var):
    '''Called when a player spawns'''

    # Is player on a team?
    if int(event_var['es_userteam']):

        # Set player's score
        set_score(event_var['userid'])


def player_death(event_var):
    '''Called when a player dies'''

    # Get the attacker
    attacker = int(event_var['attacker'])

    # Do we need to check the attacker's score?
    if attacker:

        # Set attacker's score
        set_score(attacker)

    # Was world damage?
    else:

        # Get the userid
        userid = event_var['userid']

        # Set the victims score, in case of suicide penalty
        set_score(userid)

        # Set the player's score, again, after a delay
        delayed(.1, set_score, userid)


def bomb_defused(event_var):
    '''Called when a player defuses the bomb'''

    # Set defuser's score
    set_score(event_var['userid'])


def bomb_exploded(event_var):
    '''Called when the bomb explodes'''

    # Set detonator's score
    set_score(event_var['userid'])


def gg_win(event_var):
    '''Called when a player wins the match'''

    # Set winner's score
    set_score(event_var['winner'])


def round_end(event_var):
    '''Called when the round ends'''

    # Set all player scores
    set_player_scores()


# =============================================================================
# >> CUSTOM/HELPER FUNCTIONS
# =============================================================================
def set_player_scores():
    # Loop through all players on the server
    for userid in getUseridList():

        # Set each player's score
        set_score(userid)

        # Set each player's score, again, after a delay
        delayed(.1, set_score, userid)


def set_score(userid):
    # Make sure the userid is an integer
    userid = int(userid)

    # Is player still on the server?
    if not exists('userid', userid):
        return

    # Get the SPE getPlayer instance
    spePlayer = getPlayer(userid)

    # Does the player's SPE instance exist?
    if spePlayer is None:
        return

    # Get the player instance
    ggPlayer = Player(userid)

    # Get the kill offset for the player
    kill_offset = spePlayer + gg_kill_offset

    # Do we need to update kills?
    if getLocVal('i', kill_offset) != ggPlayer.level:

        # Set the player's score to their current level
        setLocVal('i', kill_offset, ggPlayer.level)

    # Do we want to set deaths to multikill amount?
    if not int(gg_scoreboard_multikill):
        return

    # Get the death offset for the player
    death_offset = spePlayer + gg_death_offset

    # Get the player's current death total
    death_total = getLocVal('i', death_offset)

    # Are we setting the score to the number of kills remaining?
    if int(gg_scoreboard_multikill) == 2:

        # Get the player's level's multikill value
        level_multikill = get_level_multikill(ggPlayer.level)

        # Get the remaining kills for the player's current level
        kills_remaining = level_multikill - ggPlayer.multikill

        # Do we need to update the player's death total?
        if death_total != kills_remaining:

            # Set the player's death total to the number of kills remaining
            setLocVal('i', death_offset, kills_remaining)

    # Do we need to update deaths to the player's current multikill value?
    elif death_total != ggPlayer.multikill:

        # Set the player's deathcount to their current multikill value
        setLocVal('i', death_offset, ggPlayer.multikill)
